@@ -1,16 +1,25 @@
-// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+// SPDX-FileCopyrightText: 2022-2024 UnionTech Software Technology Co., Ltd.
 //
 // SPDX-License-Identifier: LGPL-3.0-or-later
 
+
 #include "dguiapplicationhelper.h"
 #include "dplatformhandle.h"
 #include "dplatformtheme.h"
 #include "dwindowmanagerhelper.h"
 
-#include <QWindow>
+#ifndef DTK_DISABLE_TREELAND
+#include "plugins/platform/treeland/dtreelandplatformwindowinterface.h"
+#endif
+
+#include <private/qwaylandwindow_p.h>
+#include <dtkcore_global.h>
+
 #include <QGuiApplication>
 #include <QDebug>
 #include <QPlatformSurfaceEvent>
+#include <QStyleHints>
+#include <QHash>
 
 DGUI_BEGIN_NAMESPACE
 
@@ -25,6 +34,8 @@ DEFINE_CONST_CHAR(netWmStates);
 DEFINE_CONST_CHAR(windowRadius);
 DEFINE_CONST_CHAR(borderWidth);
 DEFINE_CONST_CHAR(borderColor);
+DEFINE_CONST_CHAR(windowEffect);
+DEFINE_CONST_CHAR(windowStartUpEffect);
 DEFINE_CONST_CHAR(shadowRadius);
 DEFINE_CONST_CHAR(shadowOffset);
 DEFINE_CONST_CHAR(shadowColor);
@@ -93,6 +104,20 @@ static void setWindowProperty(QWindow *window, const char *name, const QVariant
     reinterpret_cast<void(*)(QWindow *, const char *, const QVariant &)>(setWindowProperty)(window, name, value);
 }
 
+#ifndef DTK_DISABLE_TREELAND
+static QHash<DPlatformHandle *, DTreeLandPlatformWindowInterface *> g_platformThemeMap;
+
+static DTreeLandPlatformWindowInterface *dPlatformWindowInterfaceByWindow(QWindow * window)
+{
+    for (auto it = g_platformThemeMap.cbegin(); it != g_platformThemeMap.cend(); ++it) {
+        if (it.value()->getWindow() == window) {
+            return it.value();
+        }
+    }
+    return nullptr;
+}
+#endif
+
 /*!
   \class Dtk::Gui::DPlatformHandle
   \inmodule dtkgui
@@ -192,12 +217,12 @@ static void setWindowProperty(QWindow *window, const char *name, const QVariant
   QWidget w;
   QPainterPath path;
   QFont font;
-  
+
   font.setPixelSize(100);
   path.addText(0, 150, font, "deepin");
-  
+
   DPlatformHandle handle(&w);
-  
+
   handle.setClipPath(path);
   w.resize(400, 200);
   w.show();
@@ -215,7 +240,7 @@ static void setWindowProperty(QWindow *window, const char *name, const QVariant
   \code
   QWidget w;
   DPlatformHandle handle(&w);
-  
+
   // 为何更好的观察效果，此处将阴影改为蓝色
   handle.setShadowColor(Qt::blue);
   w.resize(400, 200);
@@ -380,11 +405,26 @@ DPlatformHandle::DPlatformHandle(QWindow *window, QObject *parent)
     : QObject(parent)
     , m_window(window)
 {
+#ifndef DTK_DISABLE_TREELAND
+    if (DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::IsWaylandPlatform)) {
+        g_platformThemeMap.insert(this, new DTreeLandPlatformWindowInterface(nullptr, window));
+    }
+#endif
+
     enableDXcbForWindow(window);
 
     window->installEventFilter(this);
 }
 
+DPlatformHandle::~DPlatformHandle()
+{
+#ifndef DTK_DISABLE_TREELAND
+    if (auto item = g_platformThemeMap.take(this)) {
+        item->deleteLater();
+    }
+#endif
+}
+
 /*!
   \brief DPlatformHandle::pluginVersion
   \return 返回dxcb插件的版本
@@ -447,26 +487,26 @@ bool DPlatformHandle::isDXcbPlatform()
   ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
   </pre>
   \endraw
-  
+
   但是，如果窗口管理器自身支持隐藏窗口标题栏，则此方法将优先调用 enableNoTitlebarForWindow 实现同样的效果。
-  
+
   例子：
   \code
   QWidget w1;
-  
+
   w1.setWindowTitle("使用系统边框的窗口");
   w1.show();
-  
+
   DMainWindow w2;
   QWidget w3;
-  
+
   w2.titlebar()->setTitle("使用DTK风格边框带标题栏的窗口");
   w3.setWindowTitle("使用DTK风格边框没有标题栏的窗口");
   w2.show();
-  
+
   DPlatformHandle::enableDXcbForWindow(&w3);
   w3.show(); // 因为这个窗口没有标题栏，所以不会显示窗口标题
-  
+
   \endcode
   \image dtk_and_system_window.jpeg
   开启了dxcb的窗口，在窗口外边缘10像素的范围按下鼠标左键可以触发改变窗口大小的行为，
@@ -480,7 +520,7 @@ bool DPlatformHandle::isDXcbPlatform()
   {
   public:
      explicit Window() {
-  
+
      }
 
   protected:
@@ -495,10 +535,10 @@ bool DPlatformHandle::isDXcbPlatform()
   w.show();
   \endcode
   将无法使用鼠标移动窗口w
-  
+
   窗口管理器（如X11平台上的Window Manager）是否支持混成会影响dxcb插件对窗口添加的默认装饰。
   \note 在 Deepin 桌面环境中，打开窗口特效则支持混成，关闭窗口特效则不支持混成
-  
+
   支持混成：
   \image enable_composite.png
   不支持混成：
@@ -604,20 +644,19 @@ static void initWindowRadius(QWindow *window)
     }
 }
 
-class Q_DECL_HIDDEN CreatorWindowEventFile : public QObject {
+class Q_DECL_HIDDEN CreatorWindowEventFilter : public QObject {
 public:
-    CreatorWindowEventFile(QObject *par= nullptr): QObject(par){}
+    CreatorWindowEventFilter(QObject *par= nullptr): QObject(par){}
 
 public:
     bool eventFilter(QObject *watched, QEvent *event) override {
         if (event->type() == QEvent::PlatformSurface) {
             QPlatformSurfaceEvent *se = static_cast<QPlatformSurfaceEvent*>(event);
             if (se->surfaceEventType() == QPlatformSurfaceEvent::SurfaceCreated) {  // 若收到此信号， 则 WinID 已被创建
-                initWindowRadius(qobject_cast<QWindow *>(watched));
-                deleteLater();
+                auto window = qobject_cast<QWindow *>(watched);
+                initWindowRadius(window);
             }
         }
-
         return QObject::eventFilter(watched, event);
     }
 };
@@ -639,9 +678,20 @@ bool DPlatformHandle::setEnabledNoTitlebarForWindow(QWindow *window, bool enable
     auto isDWaylandPlatform = [] {
         return qApp->platformName() == "dwayland" || qApp->property("_d_isDwayland").toBool();
     };
-    if (!(isDXcbPlatform() || isDWaylandPlatform()))
+    if (!(isDXcbPlatform() || isDWaylandPlatform() || DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::IsWaylandPlatform)))
         return false;
 
+#ifndef DTK_DISABLE_TREELAND
+    if (window && DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::IsWaylandPlatform)) {
+
+        auto handle = dPlatformWindowInterfaceByWindow(window);
+        if (handle) {
+            handle->setEnabledNoTitlebar(enable);
+        }
+        return true;
+    }
+#endif
+
     if (isEnabledNoTitlebar(window) == enable)
         return true;
 
@@ -657,7 +707,7 @@ bool DPlatformHandle::setEnabledNoTitlebarForWindow(QWindow *window, bool enable
             if (window->handle()) {
                 initWindowRadius(window);
             } else {
-                window->installEventFilter(new CreatorWindowEventFile(window));
+                window->installEventFilter(new CreatorWindowEventFilter(window));
             }
         }
 
@@ -712,7 +762,7 @@ inline DPlatformHandle::WMBlurArea operator *(const DPlatformHandle::WMBlurArea
   QWindow w;
   QVector<DPlatformHandle::WMBlurArea> area_list;
   DPlatformHandle::WMBlurArea area;
-  
+
   area.x = 50;
   area.y = 50;
   area.width = 200;
@@ -720,16 +770,16 @@ inline DPlatformHandle::WMBlurArea operator *(const DPlatformHandle::WMBlurArea
   area.xRadius = 10;
   area.yRaduis = 10;
   area_list.append(area);
-  
+
   DPlatformHandle::setWindowBlurAreaByWM(&w, area_list);
-  
+
   QSurfaceFormat format = w.format();
   format.setAlphaBufferSize(8);
-  
+
   w.setFormat(format);
   w.resize(300, 300);
   w.show();
-  
+
   \endcode
   \image blur_window_demo1.png
   \a window 目标窗口对象
@@ -821,21 +871,21 @@ inline QPainterPath operator *(const QPainterPath &path, qreal scale)
   QList<QPainterPath> path_list;
   QPainterPath path;
   QFont font;
-  
+
   font.setPixelSize(100);
   font.setBold(true);
   path.addText(0, 150, font, "deepin");
   path_list.append(path);
-  
+
   DPlatformHandle::setWindowBlurAreaByWM(&w, path_list);
-  
+
   QSurfaceFormat format = w.format();
   format.setAlphaBufferSize(8);
-  
+
   w.setFormat(format);
   w.resize(300, 300);
   w.show();
-  
+
   \endcode
   \image blur_window_demo2.png
   \a window 目标窗口对象
@@ -909,19 +959,19 @@ bool DPlatformHandle::setWindowBlurAreaByWM(QWindow *window, const QList<QPainte
   QRect area;
   WallpaperScaleMode sMode
   WallpaperFillMode fMode
-  
+
   area.setRect(50, 50, 200, 200);
   bMode = WallpaperScaleFlag::FollowWindow | WallpaperFillFlag::PreserveAspectCrop;
-  
+
   DPlatformHandle::setWindowWallpaperParaByWM(&w, area, bMode);
-  
+
   QSurfaceFormat format = w.format();
   format.setAlphaBufferSize(8);
-  
+
   w.setFormat(format);
   w.resize(300, 300);
   w.show();
-  
+
   \endcode
   \a window 目标窗口对象
   \a area 壁纸区域，此区域范围内的窗口背景将填充为用户设置的当前工作区窗口壁纸
@@ -1088,6 +1138,16 @@ QColor DPlatformHandle::shadowColor() const
     return qvariant_cast<QColor>(m_window->property(_shadowColor));
 }
 
+DPlatformHandle::EffectScene DPlatformHandle::windowEffect()
+{
+    return qvariant_cast<EffectScene>(m_window->property(_windowEffect));
+}
+
+DPlatformHandle::EffectType DPlatformHandle::windowStartUpEffect()
+{
+    return qvariant_cast<EffectType>(m_window->property(_windowStartUpEffect));
+}
+
 QPainterPath DPlatformHandle::clipPath() const
 {
     return qvariant_cast<QPainterPath>(m_window->property(_clipPath));
@@ -1164,6 +1224,16 @@ void DPlatformHandle::setBorderColor(const QColor &borderColor)
     setWindowProperty(m_window, _borderColor, QVariant::fromValue(borderColor));
 }
 
+void DPlatformHandle::setWindowEffect(DPlatformHandle::EffectScenes effectScene)
+{
+    setWindowProperty(m_window, _windowEffect, static_cast<quint32>(effectScene));
+}
+
+void DPlatformHandle::setWindowStartUpEffect(DPlatformHandle::EffectTypes effectType)
+{
+    setWindowProperty(m_window, _windowStartUpEffect, static_cast<quint32>(effectType));
+}
+
 void DPlatformHandle::setShadowRadius(int shadowRadius)
 {
     setWindowProperty(m_window, _shadowRadius, shadowRadius);
@@ -1206,6 +1276,12 @@ void DPlatformHandle::setEnableSystemMove(bool enableSystemMove)
 
 void DPlatformHandle::setEnableBlurWindow(bool enableBlurWindow)
 {
+#ifndef DTK_DISABLE_TREELAND
+    if (DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::IsWaylandPlatform)) {
+        g_platformThemeMap.value(this)->setEnableBlurWindow(enableBlurWindow);
+        return;
+    }
+#endif
     setWindowProperty(m_window, _enableBlurWindow, enableBlurWindow);
 }
 
